@@ -19,6 +19,44 @@ from ..types import ConfigDict as ConfigDictType
 from ..types import FilePath, LogLevel, ModulePath, ServiceName, ServiceType, Version
 
 
+class AgentType(str, Enum):
+    """Agent execution types."""
+
+    REACTIVE = "reactive"
+    ITERATIVE = "iterative"
+
+
+class MemoryConfig(BaseModel):
+    """Memory configuration for agents."""
+
+    persistence: bool = Field(default=True, description="Enable memory persistence")
+    max_entries: int = Field(default=1000, description="Maximum memory entries")
+    ttl_hours: int = Field(default=24, description="Memory TTL in hours")
+
+
+class IterativeConfig(BaseModel):
+    """Configuration for iterative agents."""
+
+    max_iterations: int = Field(default=10, ge=1, le=100, description="Maximum iterations per task")
+    reflection_interval: int = Field(default=1, ge=1, description="Reflect every N iterations")
+    require_explicit_completion: bool = Field(default=True, description="Require explicit completion")
+    timeout_minutes: int = Field(default=30, ge=1, description="Timeout in minutes")
+    completion_confidence_threshold: float = Field(
+        default=0.8, ge=0.0, le=1.0, description="Minimum confidence threshold (0.0-1.0) for goal completion"
+    )
+
+
+class BaseAgent(BaseModel):
+    model_config = {
+        "arbitrary_types_allowed": True,
+        "extra": "allow",
+    }
+
+    agent_name: str = Field(description="The name of the agent.")
+    description: str = Field(description="A brief description of the agent's purpose.")
+    content_types: list[str] = Field(description="Supported content types.")
+
+
 class EnvironmentVariable(BaseModel):
     name: str = Field(..., description="Environment variable name")
     default: str | None = Field(None, description="Default value if not set")
@@ -38,33 +76,33 @@ class LogFormat(str, Enum):
 
 
 class LoggingConsoleConfig(BaseModel):
-    enabled: bool = Field(True, description="Enable console logging")
-    colors: bool = Field(True, description="Enable colored output")
-    show_time: bool = Field(True, description="Show timestamps")
-    show_level: bool = Field(True, description="Show log level")
+    enabled: bool = Field(default=True, description="Enable console logging")
+    colors: bool = Field(default=True, description="Enable colored output")
+    show_time: bool = Field(default=True, description="Show timestamps")
+    show_level: bool = Field(default=True, description="Show log level")
 
 
 class LoggingFileConfig(BaseModel):
-    enabled: bool = Field(False, description="Enable file logging")
-    path: FilePath = Field("logs/agentup.log", description="Log file path")
-    max_size: int = Field(10 * 1024 * 1024, description="Max file size in bytes")
-    backup_count: int = Field(5, description="Number of backup files to keep")
-    rotation: Literal["size", "time", "never"] = Field("size", description="Rotation strategy")
+    enabled: bool = Field(default=False, description="Enable file logging")
+    path: FilePath = Field(default="logs/agentup.log", description="Log file path")
+    max_size: int = Field(default=10 * 1024 * 1024, description="Max file size in bytes")
+    backup_count: int = Field(default=5, description="Number of backup files to keep")
+    rotation: Literal["size", "time", "never"] = Field(default="size", description="Rotation strategy")
 
 
 class LoggingConfig(BaseModel):
-    enabled: bool = Field(True, description="Enable logging system")
-    level: LogLevel = Field("INFO", description="Global log level")
-    format: LogFormat = Field(LogFormat.TEXT, description="Log output format")
+    enabled: bool = Field(default=True, description="Enable logging system")
+    level: LogLevel = Field(default="INFO", description="Global log level")
+    format: LogFormat = Field(default=LogFormat.TEXT, description="Log output format")
 
     # Output destinations
-    console: LoggingConsoleConfig = Field(default_factory=LoggingConsoleConfig)
-    file: LoggingFileConfig = Field(default_factory=LoggingFileConfig)
+    console: LoggingConsoleConfig = Field(default_factory=lambda: LoggingConsoleConfig())
+    file: LoggingFileConfig = Field(default_factory=lambda: LoggingFileConfig())
 
     # Advanced configuration
-    correlation_id: bool = Field(True, description="Include correlation IDs")
-    request_logging: bool = Field(True, description="Log HTTP requests")
-    structured_data: bool = Field(False, description="Include structured metadata")
+    correlation_id: bool = Field(default=True, description="Include correlation IDs")
+    request_logging: bool = Field(default=True, description="Log HTTP requests")
+    structured_data: bool = Field(default=False, description="Include structured metadata")
 
     # Module-specific log levels
     modules: dict[str, LogLevel] = Field(default_factory=dict)
@@ -176,6 +214,11 @@ class MCPServerConfig(BaseModel):
     # Tool permissions (REQUIRED for security)
     tool_scopes: dict[str, list[str]] = Field(..., description="Tool name to required scopes mapping (REQUIRED)")
 
+    # Multi-agent discovery
+    expose_as_skills: bool = Field(
+        False, description="Expose MCP tools as skills in AgentCard for multi-agent discovery"
+    )
+
     @model_validator(mode="after")
     def validate_server_config(self) -> MCPServerConfig:
         if self.transport == "stdio":
@@ -197,17 +240,17 @@ class MCPServerConfig(BaseModel):
 
 
 class MCPConfig(BaseModel):
-    enabled: bool = Field(False, description="Enable MCP support")
+    enabled: bool = Field(default=False, description="Enable MCP support")
 
     # Client configuration
-    client_enabled: bool = Field(True, description="Enable MCP client")
-    client_timeout: int = Field(30, description="Client timeout in seconds")
-    client_retry_attempts: int = Field(3, description="Client retry attempts")
+    client_enabled: bool = Field(default=True, description="Enable MCP client")
+    client_timeout: int = Field(default=30, description="Client timeout in seconds")
+    client_retry_attempts: int = Field(default=3, description="Client retry attempts")
 
     # Server configuration
-    server_enabled: bool = Field(False, description="Enable MCP server")
-    server_host: str = Field("localhost", description="Server host")
-    server_port: int = Field(8080, description="Server port")
+    server_enabled: bool = Field(default=False, description="Enable MCP server")
+    server_host: str = Field(default="localhost", description="Server host")
+    server_port: int = Field(default=8080, description="Server port")
 
     # Server configurations
     servers: list[MCPServerConfig] = Field(default_factory=list, description="MCP servers")
@@ -264,12 +307,28 @@ class OAuth2Config(BaseModel):
 
 
 class SecurityConfig(BaseModel):
-    enabled: bool = Field(True, description="Enable security features")
+    enabled: bool = Field(default=True, description="Enable security features")
     auth: dict[str, ApiKeyConfig | BearerConfig | JWTConfig | OAuth2Config] = Field(
         default_factory=dict, description="Authentication configuration by type"
     )
     scope_hierarchy: dict[str, list[str]] = Field(default_factory=dict, description="Scope hierarchy configuration")
 
+    @field_validator("scope_hierarchy", mode="before")
+    @classmethod
+    def validate_scope_hierarchy(cls, v):
+        """Handle None values for scope_hierarchy."""
+        if v is None:
+            return {}
+        return v
+
+    @field_validator("auth", mode="before")
+    @classmethod
+    def validate_auth(cls, v):
+        """Handle None values for auth."""
+        if v is None:
+            return {}
+        return v
+
 
 class PluginCapabilityConfig(BaseModel):
     capability_id: str = Field(..., description="Capability identifier")
@@ -278,12 +337,15 @@ class PluginCapabilityConfig(BaseModel):
     required_scopes: list[str] = Field(default_factory=list, description="Required scopes")
     enabled: bool = Field(True, description="Whether capability is enabled")
     config: ConfigDictType = Field(default_factory=dict, description="Capability-specific config")
-    middleware_override: list[dict[str, Any]] | None = Field(None, description="Override middleware configuration")
+    capability_override: list[dict[str, Any]] | None = Field(
+        None, description="Capability-specific middleware overrides"
+    )
 
 
 class PluginConfig(BaseModel):
-    plugin_id: str = Field(..., description="Plugin identifier")
-    name: str | None = Field(None, description="Plugin name")
+    """Legacy plugin configuration model - use IntentConfig for new configurations"""
+
+    name: str = Field(..., description="Plugin name/identifier")
     description: str | None = Field(None, description="Plugin description")
     enabled: bool = Field(True, description="Whether plugin is enabled")
     version: Version | None = Field(None, description="Plugin version constraint")
@@ -295,14 +357,14 @@ class PluginConfig(BaseModel):
     priority: int = Field(50, description="Plugin initialization priority (lower = earlier)")
     # Default settings applied to all capabilities
     default_scopes: list[str] = Field(default_factory=list, description="Default scopes")
-    middleware: list[dict[str, Any]] | None = Field(None, description="Middleware configuration")
+    plugin_override: list[dict[str, Any]] | None = Field(None, description="Plugin-level middleware overrides")
     config: ConfigDictType = Field(default_factory=dict, description="Plugin configuration")
 
-    @field_validator("plugin_id")
+    @field_validator("name")
     @classmethod
-    def validate_plugin_id(cls, v: str) -> str:
+    def validate_name(cls, v: str) -> str:
         if not v or not v.replace("_", "").replace("-", "").replace(".", "").isalnum():
-            raise ValueError("Plugin ID must be alphanumeric with hyphens, underscores, and dots")
+            raise ValueError("Plugin name must be alphanumeric with hyphens, underscores, and dots")
         return v
 
     @computed_field  # Modern Pydantic v2 computed property
@@ -318,12 +380,12 @@ class PluginConfig(BaseModel):
     @computed_field
     @property
     def display_name(self) -> str:
-        return self.name or self.plugin_id
+        return self.name
 
     @computed_field
     @property
     def has_middleware(self) -> bool:
-        return self.middleware is not None and len(self.middleware) > 0
+        return self.plugin_override is not None and len(self.plugin_override or []) > 0
 
     @computed_field
     @property
@@ -342,8 +404,8 @@ class PluginConfig(BaseModel):
         score += min(0.4, len(self.capabilities) / 10 * 0.4)
 
         # Middleware complexity (0.0 to 0.2)
-        if self.has_middleware:
-            score += min(0.2, len(self.middleware) / 5 * 0.2)
+        if self.has_middleware and self.plugin_override:
+            score += min(0.2, len(self.plugin_override) / 5 * 0.2)
 
         # Scope count (0.0 to 0.2)
         total_scopes = len(self.total_required_scopes)
@@ -363,12 +425,24 @@ class PluginsConfig(BaseModel):
     plugins: list[PluginConfig] = Field(default_factory=list, description="Plugin configurations")
 
 
+class AIProviderConfig(BaseModel):
+    provider: str = Field(..., description="AI provider name (e.g., openai, anthropic)")
+    api_key: str | None = Field(None, description="API key for the AI provider")
+    model: str = Field(..., description="AI model to use")
+    stream: bool = Field(False, description="Enable streaming responses")
+    chunk_size: int = Field(50, description="Chunk size for non-streaming fallback")
+    stream_timeout: int = Field(30, description="Timeout for streaming responses")
+    temperature: float = Field(0.7, description="Sampling temperature")
+    max_tokens: int = Field(1000, description="Maximum number of tokens to generate")
+    top_p: float = Field(1.0, description="Top-p sampling")
+
+
 class MiddlewareConfig(BaseModel):
-    enabled: bool = Field(True, description="Enable middleware system")
+    enabled: bool = Field(default=True, description="Enable middleware system")
 
     # Rate limiting
     rate_limiting: dict[str, Any] = Field(
-        default_factory=lambda: {"enabled": True, "requests_per_minute": 60, "burst_size": 10}
+        default_factory=lambda: {"enabled": True, "requests_per_minute": 60, "burst_size": 72}
     )
 
     # Caching
@@ -383,22 +457,22 @@ class MiddlewareConfig(BaseModel):
 
 
 class APIConfig(BaseModel):
-    enabled: bool = Field(True, description="Enable API server")
-    host: str = Field("127.0.0.1", description="Server host")
-    port: int = Field(8000, description="Server port")
+    enabled: bool = Field(default=True, description="Enable API server")
+    host: str = Field(default="127.0.0.1", description="Server host")
+    port: int = Field(default=8000, description="Server port")
 
     # Server settings
-    workers: int = Field(1, description="Number of workers")
-    reload: bool = Field(False, description="Enable auto-reload")
-    debug: bool = Field(False, description="Enable debug mode")
+    workers: int = Field(default=1, description="Number of workers")
+    reload: bool = Field(default=False, description="Enable auto-reload")
+    debug: bool = Field(default=False, description="Enable debug mode")
 
     # Request handling
-    max_request_size: int = Field(16 * 1024 * 1024, description="Max request size in bytes")
-    request_timeout: int = Field(30, description="Request timeout in seconds")
-    keepalive_timeout: int = Field(5, description="Keep-alive timeout in seconds")
+    max_request_size: int = Field(default=16 * 1024 * 1024, description="Max request size in bytes")
+    request_timeout: int = Field(default=30, description="Request timeout in seconds")
+    keepalive_timeout: int = Field(default=5, description="Keep-alive timeout in seconds")
 
     # CORS settings
-    cors_enabled: bool = Field(True, description="Enable CORS")
+    cors_enabled: bool = Field(default=True, description="Enable CORS")
     cors_origins: list[str] = Field(default_factory=lambda: ["*"], description="Allowed origins")
     cors_methods: list[str] = Field(
         default_factory=lambda: ["GET", "POST", "PUT", "DELETE"], description="Allowed methods"
@@ -425,6 +499,15 @@ class AgentConfig(BaseModel):
     description: str = Field("AI agent powered by AgentUp", description="Agent description")
     version: Version = Field("1.0.0", description="Agent version")
 
+    # Agent execution configuration
+    agent_type: AgentType = Field(AgentType.REACTIVE, description="Type of agent execution (reactive or iterative)")
+    memory_config: MemoryConfig = Field(
+        default_factory=lambda: MemoryConfig(), description="Memory configuration for agents"
+    )
+    iterative_config: IterativeConfig = Field(
+        default_factory=lambda: IterativeConfig(), description="Configuration for iterative agents"
+    )
+
     # Add property for backward compatibility
     @property
     def name(self) -> str:
@@ -441,12 +524,12 @@ class AgentConfig(BaseModel):
     mcp_shutdown_path: ModulePath | None = Field(None, description="MCP shutdown module path")
 
     # Configuration sections
-    logging: LoggingConfig = Field(default_factory=LoggingConfig)
-    api: APIConfig = Field(default_factory=APIConfig)
-    security: SecurityConfig = Field(default_factory=SecurityConfig)
-    plugins: list[PluginConfig] = Field(default_factory=list, description="Plugin configurations")
-    middleware: MiddlewareConfig = Field(default_factory=MiddlewareConfig)
-    mcp: MCPConfig = Field(default_factory=MCPConfig)
+    logging: LoggingConfig = Field(default_factory=lambda: LoggingConfig())
+    api: APIConfig = Field(default_factory=lambda: APIConfig())
+    security: SecurityConfig = Field(default_factory=lambda: SecurityConfig())
+    plugins: dict[str, Any] = Field(default_factory=dict, description="Plugin configurations (dictionary format)")
+    middleware: MiddlewareConfig = Field(default_factory=lambda: MiddlewareConfig())
+    mcp: MCPConfig = Field(default_factory=lambda: MCPConfig())
 
     # AI configuration
     ai: dict[str, Any] = Field(default_factory=dict, description="AI settings")
@@ -483,10 +566,14 @@ class AgentConfig(BaseModel):
     @field_validator("version")
     @classmethod
     def validate_version(cls, v: str) -> str:
-        import re
+        import semver
 
-        if not re.match(r"^\d+\.\d+\.\d+(?:-[\w.-]+)?$", v):
-            raise ValueError("Version must follow semantic versioning (e.g., 1.0.0)")
+        try:
+            semver.Version.parse(v)
+        except ValueError:
+            raise ValueError(
+                "Version must follow semantic versioning (e.g., 1.0.0, 1.2.3-alpha.1, 1.0.0+build.123)"
+            ) from None
         return v
 
     @computed_field  # Modern Pydantic v2 computed property
@@ -523,7 +610,8 @@ class AgentConfig(BaseModel):
     def validate_mcp_consistency(self) -> AgentConfig:
         if self.mcp_enabled:
             if not self.mcp.enabled:
-                self.mcp = MCPConfig(enabled=True)
+                # MCPConfig will use its own field defaults
+                self.mcp.enabled = True
         return self
 
 
@@ -547,7 +635,7 @@ class ConfigurationSettings(BaseSettings):
     # Security settings
     SECRET_KEY: str | None = Field(None, description="Application secret key")
 
-    model_config = ConfigDict(env_prefix="AGENTUP_", case_sensitive=True)
+    model_config = {"env_prefix": "AGENTUP_", "case_sensitive": True}
 
     def create_directories(self) -> None:
         directories = [self.DATA_DIR, self.LOGS_DIR, self.PLUGINS_DIR]
@@ -582,6 +670,7 @@ def expand_env_vars(value: Any) -> Any:
 # Re-export key models
 __all__ = [
     "AgentConfig",
+    "BaseAgent",
     "ServiceConfig",
     "LoggingConfig",
     "APIConfig",
@@ -593,4 +682,7 @@ __all__ = [
     "ConfigurationSettings",
     "EnvironmentVariable",
     "expand_env_vars",
+    "AgentType",
+    "MemoryConfig",
+    "IterativeConfig",
 ]
