@@ -1,4 +1,4 @@
-from collections.abc import AsyncIterator, Callable
+from collections.abc import Callable
 from typing import Any
 
 import structlog
@@ -7,7 +7,6 @@ from a2a.types import Task
 from agent.services import get_services
 from agent.services.llm.manager import LLMManager
 from agent.state.conversation import ConversationManager
-from agent.utils.messages import MessageProcessor
 
 from .function_executor import FunctionExecutor
 
@@ -28,6 +27,8 @@ class FunctionRegistry:
         self._handlers: dict[str, Callable] = {}
         self._mcp_tools: dict[str, dict[str, Any]] = {}
         self._mcp_client = None
+        # Cache for scope-filtered tools (never expires - registry is static after startup)
+        self._scope_filtered_cache: dict[str, list[dict[str, Any]]] = {}
 
     def register_function(self, name: str, handler: Callable, schema: dict[str, Any]):
         if name in self._functions:
@@ -77,18 +78,26 @@ class FunctionRegistry:
         Returns:
             list[dict[str, Any]]: List of tool schemas filtered by user permissions
         """
+        # If no scopes provided, return empty list for security
+        if not user_scopes:
+            return []
+
+        # Create cache key from sorted scopes
+        scopes_key = ",".join(sorted(user_scopes))
+
+        # Check cache (never expires - function registry is static after startup)
+        if scopes_key in self._scope_filtered_cache:
+            logger.debug(f"Using cached function schemas for scopes: {len(user_scopes)} scopes")
+            return self._scope_filtered_cache[scopes_key]
+
+        scope_service = None
         try:
             from agent.security.scope_service import get_scope_service
 
-            # If no scopes provided, return empty list for security
-            if not user_scopes:
-                return []
-
             scope_service = get_scope_service()
             if not scope_service._hierarchy:
-                # If no scope hierarchy available, return all tools (fallback behavior)
-                logger.warning("No scope hierarchy available - falling back to all tools")
-                return self.get_function_schemas()
+                logger.error("No scope hierarchy available - denying all tool access")
+                return []
 
             available_tools = []
             logger.debug("Using optimized scope service for tool filtering")
@@ -96,6 +105,13 @@ class FunctionRegistry:
             # Start request-scoped cache
             _cache = scope_service.start_request_cache()
 
+            # Check local functions (including system capabilities) first
+            try:
+                available_tools.extend(self._get_local_functions(scope_service, user_scopes))
+            except Exception as e:
+                logger.warning(f"Failed to filter local functions by scopes: {e}")
+                # Security: Do NOT fallback to all tools - continue with empty local tools list
+
             # Check local plugin capabilities
             try:
                 available_tools.extend(self._get_plugin_tools(scope_service, user_scopes))
@@ -131,20 +147,22 @@ class FunctionRegistry:
             # Clean up request cache
             scope_service.clear_request_cache()
 
+            # Cache the results (permanent - registry is static after startup)
+            self._scope_filtered_cache[scopes_key] = final_tools
+
             return final_tools
 
         except Exception as e:
             logger.error(f"Failed to filter tools by user scopes: {e}")
             # Clean up cache on error
-            try:
-                scope_service.clear_request_cache()
-            except Exception:
-                # Security fallback: return empty list rather than all tools
-                pass  # nosec
+            if scope_service:
+                try:
+                    scope_service.clear_request_cache()
+                except Exception:
+                    pass  # nosec
             return []
 
     def _get_plugin_tools(self, scope_service, user_scopes: set[str]) -> list[dict[str, Any]]:
-        from agent.config import Config
         from agent.plugins.integration import get_plugin_adapter
         from agent.security.audit_logger import get_security_audit_logger
 
@@ -170,58 +188,65 @@ class FunctionRegistry:
 
             user_id = f"session_{hashlib.sha256(uuid.uuid4().bytes).hexdigest()}"
 
-        logger.debug(
-            f"Plugin capability filtering - adapter: {plugin_adapter is not None}, config: {Config is not None}"
-        )
-
         if not plugin_adapter:
             return tools
 
-        configured_plugins = Config.plugins
-        logger.debug(f"Found {len(configured_plugins)} configured plugins")
+        # Get plugins from the integration system instead of old config
+        from agent.plugins.integration import get_plugin_registry_instance
 
-        for plugin_config in configured_plugins:
-            plugin_id = plugin_config.plugin_id
-            if not plugin_id:
+        plugin_registry = get_plugin_registry_instance()
+
+        if not plugin_registry:
+            return tools
+
+        # Track which functions we've already seen to avoid duplicates
+        seen_functions = set()
+
+        # First, collect all functions already registered in the function registry
+        # These were registered during initialization via register_ai_functions_from_capabilities
+        for function_name in self._functions.keys():
+            seen_functions.add(function_name)
+
+        for plugin_name, plugin_instance in plugin_registry.plugins.items():
+            if not plugin_name:
                 continue
 
-            logger.debug(f"Processing plugin: {plugin_id}")
+            # Get capabilities from modern decorator-based plugins
+            try:
+                plugin_capabilities = plugin_instance.get_capability_definitions()
 
-            # Handle new capability-based config structure
-            if plugin_config.capabilities:
-                capabilities_list = plugin_config.capabilities
-                logger.debug(f"Plugin '{plugin_id}' has {len(capabilities_list)} capabilities defined")
-
-                for capability_config in capabilities_list:
-                    capability_id = capability_config.capability_id
-                    required_scopes = capability_config.required_scopes or []
+                for capability_def in plugin_capabilities:
+                    capability_id = capability_def.id
+                    # Get required scopes from the @capability decorator
+                    required_scopes = capability_def.required_scopes
 
                     # Use centralized scope validation
                     result = scope_service.validate_multiple_scopes(user_scopes, required_scopes)
-                    logger.debug(f"Capability '{capability_id}' access: {result.has_access}")
 
                     if result.has_access:
                         # Get AI functions for this capability
                         ai_functions = plugin_adapter.get_ai_functions(capability_id)
                         for ai_function in ai_functions:
+                            # Skip if this function was already registered
+                            if ai_function.name in seen_functions:
+                                logger.debug(f"Skipping duplicate plugin function: {ai_function.name}")
+                                continue
+
                             tool_spec = {
                                 "name": ai_function.name,
                                 "description": ai_function.description,
                                 "parameters": ai_function.parameters,
                             }
                             tools.append(tool_spec)
-                            logger.debug(f"Granted AI function '{ai_function.name}' for capability '{capability_id}'")
+                            seen_functions.add(ai_function.name)
                     else:
                         # Log function access denied for security audit
                         ai_functions = plugin_adapter.get_ai_functions(capability_id)
                         for ai_function in ai_functions:
                             audit_logger.log_function_access_denied(user_id, ai_function.name, len(required_scopes))
-            else:
-                # Handle legacy config structure
-                logger.warning(f"Plugin '{plugin_id}' using legacy config structure - no scope enforcement")
-                audit_logger.log_configuration_error(
-                    f"plugin_{plugin_id}", "legacy_config_no_scope_enforcement", {"plugin_id": plugin_id}
-                )
+            except Exception as e:
+                logger.warning(f"Failed to get capabilities for plugin '{plugin_name}': {e}")
+                continue
         return tools
 
     def _get_mcp_tools(self, scope_service, user_scopes: set[str]) -> list[dict[str, Any]]:
@@ -253,6 +278,38 @@ class FunctionRegistry:
 
         return tools
 
+    def _get_local_functions(self, scope_service, user_scopes: set[str]) -> list[dict[str, Any]]:
+        """Get local functions (including system capabilities) filtered by user scopes.
+
+        Local functions include system capabilities like mark_goal_complete that are
+        registered directly with the function registry.
+        """
+        tools = []
+
+        # For system capabilities, we need minimal scopes since they're built-in tools
+        # System tools like mark_goal_complete should be available to users with basic access
+        system_tool_scopes = ["system:read"]  # Minimal scope requirement for system tools
+
+        for function_name, function_schema in self._functions.items():
+            # System capabilities get minimal scope requirements
+            if function_name == "mark_goal_complete":
+                required_scopes = system_tool_scopes
+                logger.debug(f"Checking system capability '{function_name}' with scopes: {required_scopes}")
+            else:
+                # Other local functions might have different scope requirements
+                # For now, treat them all as system tools
+                required_scopes = system_tool_scopes
+
+            # Use centralized scope validation
+            result = scope_service.validate_multiple_scopes(user_scopes, required_scopes)
+            if result.has_access:
+                tools.append(function_schema)
+                logger.debug(f"Granted local function '{function_name}' to user")
+            else:
+                logger.debug(f"Denied local function '{function_name}' - insufficient scopes")
+
+        return tools
+
     def get_handler(self, function_name: str) -> Callable | None:
         return self._handlers.get(function_name)
 
@@ -263,13 +320,13 @@ class FunctionRegistry:
 
     async def register_mcp_client(self, mcp_client) -> None:
         # CONDITIONAL_MCP_IMPORTS
-        logger.info(f"Registering MCP client, initialized: {mcp_client.is_initialized if mcp_client else False}")
+        logger.debug(f"Registering MCP client, initialized: {mcp_client.is_initialized if mcp_client else False}")
         self._mcp_client = mcp_client
 
         if mcp_client and mcp_client.is_initialized:
             # Get available MCP tools
             mcp_tools = await mcp_client.get_available_tools()
-            logger.info(f"Got {len(mcp_tools)} MCP tools from client")
+            logger.debug(f"Got {len(mcp_tools)} MCP tools from client")
 
             for tool_schema in mcp_tools:
                 original_name = tool_schema.get("name", "unknown")
@@ -277,13 +334,17 @@ class FunctionRegistry:
                 # Replace colons with underscores: "filesystem:read_file" -> "filesystem_read_file"
                 function_name = original_name.replace(":", "_")
 
-                # Store with the cleaned name but keep original info
-                cleaned_schema = tool_schema.copy()
-                cleaned_schema["name"] = function_name
-                cleaned_schema["original_name"] = original_name  # Keep for MCP calls
+                # Only register if not already registered by register_mcp_tool (which has better scope handling)
+                if function_name not in self._mcp_tools:
+                    # Store with the cleaned name but keep original info
+                    cleaned_schema = tool_schema.copy()
+                    cleaned_schema["name"] = function_name
+                    cleaned_schema["original_name"] = original_name  # Keep for MCP calls
 
-                self._mcp_tools[function_name] = cleaned_schema
-                logger.info(f"Registered MCP tool in function registry: {original_name} -> {function_name}")
+                    self._mcp_tools[function_name] = cleaned_schema
+                    logger.debug(f"Registered MCP tool in function registry: {original_name} -> {function_name}")
+                else:
+                    logger.debug(f"Skipping MCP tool '{function_name}' - already registered with scope enforcement")
         else:
             logger.warning(
                 f"Cannot register MCP client - client: {mcp_client is not None}, initialized: {mcp_client.is_initialized if mcp_client else False}"
@@ -324,40 +385,21 @@ class FunctionDispatcher:
 
         self.streaming_handler = StreamingHandler(function_registry, self.conversation_manager)
 
-    async def process_task(self, task: Task) -> str:
+    async def process_task(self, task: Task, auth_result=None) -> str | dict[str, Any]:
         """Process A2A task using LLM intelligence.
 
         Args:
             task: A2A Task object
+            auth_result: Optional authentication result (will use current context if not provided)
 
         Returns:
-            str: Response content for A2A message
+            str | dict[str, Any]: Response content for A2A message, or completion data dict for iterative strategy
         """
         try:
-            # Extract user message from A2A task (with multi-modal support)
-            user_message = self._extract_user_message_full(task)
-            if not user_message:
+            # Validate we have a valid task with messages
+            if not hasattr(task, "history") or not task.history:
                 return "I didn't receive any message to process."
 
-            # Extract text from A2A parts or content for backwards compatibility
-            if isinstance(user_message, dict):
-                if "parts" in user_message:
-                    # Extract text from A2A parts using proper structure
-                    user_input = ""
-                    for part in user_message["parts"]:
-                        if hasattr(part, "root") and hasattr(part.root, "kind"):
-                            if part.root.kind == "text" and hasattr(part.root, "text"):
-                                user_input += part.root.text
-                        elif hasattr(part, "text"):
-                            user_input += part.text
-                        elif isinstance(part, dict) and "text" in part:
-                            user_input += part["text"]
-                else:
-                    # Fallback to content field
-                    user_input = user_message.get("content", "")
-            else:
-                user_input = str(user_message)
-
             # Get LLM service with automatic provider selection
             services = get_services()
             llm = await LLMManager.get_llm_service(services)
@@ -368,38 +410,27 @@ class FunctionDispatcher:
                 logger.warning("2. Required API keys are set in environment variables")
                 logger.warning("3. Service initialization completed successfully")
                 logger.warning("Falling back to basic response")
+
+                # Extract user input for fallback
+                user_input = self._get_latest_user_input(task)
                 return self._fallback_response(user_input)
 
-            # Get conversation context
+            # Prepare LLM conversation from A2A task history and stored state
             try:
-                # Use context_id if available, otherwise use task ID
-                # This allows for better conversation management across tasks
-                logger.debug(f"Using context ID: {getattr(task, 'context_id', task.id)}")
-                context_id = getattr(task, "context_id", task.id)
-            except AttributeError:
-                logger.warning("Task does not have context_id, using task ID instead")
-                context_id = task.id
-
-            try:
-                conversation = self.conversation_manager.get_conversation_history(context_id)
-            except KeyError:
-                logger.warning(f"No conversation history found for context ID: {context_id}, starting fresh")
-                conversation = []
-
-            # Prepare LLM conversation with system prompt and function definitions
-            try:
-                logger.debug(f"Preparing conversation for LLM with user message: {user_message}")
-                messages = await self.conversation_manager.prepare_llm_conversation(user_message, conversation)
+                logger.debug("Preparing conversation for LLM from A2A task history and stored state")
+                messages = await self.conversation_manager.prepare_llm_conversation(task)
             except Exception as e:
                 logger.error(f"Error preparing conversation for LLM: {e}", exc_info=True)
                 return f"I encountered an error preparing your request: {str(e)}"
 
             # Get available functions filtered by user scopes
             try:
-                # Try to get user scopes from authentication context
-                from agent.security.context import get_current_auth
+                # Use provided auth_result or try to get from authentication context
+                if auth_result is None:
+                    from agent.security.context import get_current_auth
+
+                    auth_result = get_current_auth()
 
-                auth_result = get_current_auth()
                 logger.debug(f"Authentication context: {auth_result is not None}")
                 if auth_result:
                     logger.debug(f"User scopes: {auth_result.scopes}")
@@ -414,13 +445,12 @@ class FunctionDispatcher:
                             f"User '{getattr(auth_result, 'user_id', 'unknown')}' has no scopes - no tools available"
                         )
                 else:
-                    # No authentication at all - fallback to all functions (for development/testing)
-                    function_schemas = self.function_registry.get_function_schemas()
-                    logger.warning("No authentication context found, using all available functions")
+                    # No authentication - no tools available
+                    function_schemas = []
+                    logger.error("No authentication context found - denying all tool access")
             except Exception as e:
                 logger.error(f"Error retrieving function schemas: {e}", exc_info=True)
-                # Fallback to all functions on error
-                function_schemas = self.function_registry.get_function_schemas()
+                function_schemas = []
             logger.info(f"Available function schemas for AI: {len(function_schemas)} functions")
             if len(function_schemas) == 0:
                 logger.warning("No function schemas available for AI - this will prevent tool calling")
@@ -451,7 +481,21 @@ class FunctionDispatcher:
             # LLM processing with function calling
             if function_schemas:
                 try:
-                    response = await LLMManager.llm_with_functions(llm, messages, function_schemas, function_executor)
+                    llm_response = await LLMManager.llm_with_functions(
+                        llm, messages, function_schemas, function_executor
+                    )
+
+                    # Check if this is a completion response for iterative strategy
+                    if llm_response.completed and llm_response.completion_data:
+                        # Return structured completion data for iterative strategy
+                        return {
+                            "completed": True,
+                            "completion_data": llm_response.completion_data,
+                            "final_response": llm_response.content,
+                        }
+
+                    # Regular response
+                    response = llm_response.content
                 except Exception as e:
                     logger.error(f"Error during LLM function calling: {e}", exc_info=True)
                     return f"I encountered an error processing your request with functions: {str(e)}"
@@ -467,9 +511,12 @@ class FunctionDispatcher:
                     logger.error(f"Error during direct LLM response: {e}", exc_info=True)
                     return f"I encountered an error processing your request: {str(e)}"
 
-            # Store AI processing state if available
+            # Store AI processing state if available (sync A2A to persistent state)
             if ai_context and ai_context_id:
                 try:
+                    # Extract latest user message for state storage
+                    user_input = self._get_latest_user_input(task)
+
                     await self._store_ai_processing_state(ai_context, ai_context_id, user_input, response)
                     logger.info(f"AI processing: Stored conversation state for context {ai_context_id}")
                 except Exception as e:
@@ -477,9 +524,6 @@ class FunctionDispatcher:
             else:
                 logger.warning("AI processing: Skipping state storage - no context available")
 
-            # Update conversation history
-            self.conversation_manager.update_conversation_history(context_id, user_input, response)
-
             return response
 
         except Exception as e:
@@ -525,25 +569,18 @@ class FunctionDispatcher:
             conversation_count += 1
             await context.set_variable(context_id, "ai_conversation_count", conversation_count)
 
-            # Store user preferences if mentioned in input
-            if "favorite" in user_input.lower() or "prefer" in user_input.lower():
-                preferences = await context.get_variable(context_id, "ai_preferences", {})
-                # Simple preference extraction
-                if "color" in user_input.lower():
-                    colors = ["red", "blue", "green", "yellow", "purple", "orange", "pink", "black", "white"]
-                    for color in colors:
-                        if color in user_input.lower():
-                            preferences["favorite_color"] = color
-                            break
-                await context.set_variable(context_id, "ai_preferences", preferences)
-                logger.info(f"AI processing: Updated preferences for {context_id}: {preferences}")
-
             # Add to conversation history
             await context.add_to_history(
-                context_id, "user", user_input, {"processing": "ai_direct", "count": conversation_count}
+                context_id,
+                "user",
+                user_input,
+                {"processing": "ai_direct", "count": conversation_count},
             )
             await context.add_to_history(
-                context_id, "assistant", response, {"processing": "ai_direct", "count": conversation_count}
+                context_id,
+                "agent",
+                response,
+                {"processing": "ai_direct", "count": conversation_count},
             )
 
             logger.info(f"AI processing: Stored state - Context: {context_id}, Count: {conversation_count}")
@@ -551,57 +588,13 @@ class FunctionDispatcher:
         except Exception as e:
             logger.error(f"Failed to store AI processing state: {e}")
 
-    def _extract_user_message(self, task: Task) -> str:
-        # Use existing MessageProcessor for A2A compliance
-        messages = MessageProcessor.extract_messages(task)
-        latest_message = MessageProcessor.get_latest_user_message(messages)
-
-        if latest_message:
-            return (
-                latest_message.get("content", "")
-                if isinstance(latest_message, dict)
-                else getattr(latest_message, "content", "")
-            )
-
-        # Fallback to task metadata
-        if hasattr(task, "metadata") and task.metadata:
-            return task.metadata.get("user_input", "")
-
-        return ""
-
-    def _extract_user_message_full(self, task: Task) -> dict[str, Any] | str:
-        # Get the latest A2A message from task history
-        if hasattr(task, "history") and task.history:
-            for message in reversed(task.history):
-                if hasattr(message, "role") and message.role == "user":
-                    # Convert A2A Message to dict format for LLM processing
-                    if hasattr(message, "parts") and message.parts:
-                        return {
-                            "role": "user",
-                            "parts": message.parts,  # Keep full A2A parts for multi-modal
-                            "message_id": getattr(message, "message_id", "unknown"),
-                        }
-
-        # Fallback to text extraction
-        user_text = self._extract_user_message(task)
-        if user_text:
-            return {"role": "user", "content": user_text}
-
-        return ""
-
-    async def process_task_streaming(self, task: Task) -> AsyncIterator[str | dict[str, Any]]:
-        async for chunk in self.streaming_handler.process_task_streaming(
-            task, LLMManager, self._extract_user_message, self._fallback_response
-        ):
-            yield chunk
-
     async def cancel_task(self, task_id: str) -> None:
         """Cancel a running task if possible.
 
         Args:
             task_id: ID of the task to cancel
         """
-        # This would need to be implemented based on your LLM provider's capabilities
+        # This would need to be implemented based on the LLM provider's capabilities
         # Some providers support cancelling ongoing requests
         logger.info(f"Cancelling task: {task_id}")
 
@@ -612,6 +605,18 @@ class FunctionDispatcher:
     def _fallback_response(self, user_input: str) -> str:
         return f"I received your message: '{user_input}'. However, my AI capabilities are currently unavailable. Please try again later."
 
+    def _get_latest_user_input(self, task: Task) -> str:
+        """Extract the latest user input from task history."""
+        from agent.state.conversation import ConversationManager
+
+        user_input = ""
+        if task.history:
+            for message in reversed(task.history):
+                if message.role == "user" and message.parts:
+                    user_input = ConversationManager.extract_text_from_parts(message.parts)
+                    break
+        return user_input
+
 
 # Decorator for registering plugins as AI functions
 def ai_function(description: str, parameters: dict[str, Any] | None = None):
@@ -624,17 +629,21 @@ def ai_function(description: str, parameters: dict[str, Any] | None = None):
 
     def decorator(func: Callable):
         # Create function schema
-        schema = {
+        schema: dict[str, Any] = {
             "name": func.__name__.replace("handle_", ""),
             "description": description,
         }
 
         if parameters:
-            schema["parameters"] = {"type": "object", "properties": parameters, "required": list(parameters.keys())}
+            schema["parameters"] = {
+                "type": "object",
+                "properties": parameters,
+                "required": list(parameters.keys()),
+            }
 
         # Store schema on function for later registration
-        func._ai_function_schema = schema
-        func._is_ai_function = True
+        func._ai_function_schema = schema  # type: ignore[attr-defined]
+        func._is_ai_function = True  # type: ignore[attr-defined]
 
         return func
 
@@ -689,7 +698,7 @@ def register_ai_functions_from_capabilities():
 
             # Get the capabilities package
             capabilities_pkg = sys.modules.get("src.agent.capabilities") or sys.modules.get(".capabilities", None)
-            if capabilities_pkg:
+            if capabilities_pkg and capabilities_pkg.__file__:
                 capabilities_dir = Path(capabilities_pkg.__file__).parent
 
                 # Find all potential capability modules
@@ -753,7 +762,7 @@ def register_ai_functions_from_capabilities():
                     )
 
                 if has_ai_flag and has_schema:
-                    schema = obj._ai_function_schema
+                    schema = obj._ai_function_schema  # type: ignore[attr-defined]
                     registry.register_function(schema["name"], obj, schema)
                     logger.debug(f"Auto-registered AI function: {schema['name']} from {name}")
                     registered_count += 1
@@ -771,49 +780,53 @@ def register_ai_functions_from_capabilities():
 
             # Get all available plugin capabilities
             available_plugins = plugin_adapter.list_available_capabilities()
-            logger.debug(f"Available plugin capabilities: {available_plugins}")
 
-            # Get configured plugins from agent config
+            # Get configured plugins from unified configuration
             try:
-                from agent.config import Config
+                from agent.config.plugin_resolver import get_plugin_resolver
 
-                configured_plugins = {plugin.plugin_id for plugin in Config.plugins}
-                logger.debug(f"Configured plugins: {configured_plugins}")
+                resolver = get_plugin_resolver()
+                if resolver:
+                    # Get enabled plugins from intent config
+                    enabled_plugins = {}
+                    for package_name, plugin_config in resolver.intent_config.plugins.items():
+                        # Handle both string and object plugin configs
+                        if isinstance(plugin_config, str):
+                            # If it's just a string, assume it's enabled
+                            enabled_plugins[package_name] = plugin_config
+                        elif hasattr(plugin_config, "enabled") and plugin_config.enabled:
+                            enabled_plugins[package_name] = plugin_config
+                    configured_plugins = set(enabled_plugins.keys())
+                else:
+                    configured_plugins = set()
             except Exception as e:
-                logger.warning(f"Could not load agent config for plugin AI functions: {e}")
+                logger.warning(f"Could not load unified plugin config for AI functions: {e}")
                 configured_plugins = set()
 
             # Build mapping of plugin names to capabilities
             plugin_to_capabilities = {}
             for capability_id in available_plugins:
                 capability_info = plugin_adapter.get_capability_info(capability_id)
-                logger.debug(f"Capability {capability_id} info: {capability_info}")
                 if capability_info and "plugin_name" in capability_info:
                     plugin_name = capability_info["plugin_name"]
                     if plugin_name not in plugin_to_capabilities:
                         plugin_to_capabilities[plugin_name] = []
                     plugin_to_capabilities[plugin_name].append(capability_id)
 
-            logger.debug(f"Plugin to capabilities mapping: {plugin_to_capabilities}")
-
             # Register AI functions only for capabilities from configured plugins
             for capability_id in available_plugins:
                 capability_info = plugin_adapter.get_capability_info(capability_id)
                 if not capability_info or "plugin_name" not in capability_info:
-                    logger.debug(f"Skipping AI functions for capability without plugin info: {capability_id}")
                     continue
 
                 plugin_name = capability_info["plugin_name"]
-                logger.debug(f"Processing capability {capability_id} from plugin {plugin_name}")
 
-                if plugin_name not in configured_plugins:
-                    logger.debug(
-                        f"Skipping AI functions for capability '{capability_id}' from unregisterd plugin '{plugin_name}'"
-                    )
+                # Normalize plugin name to match configuration format (hyphens)
+                normalized_plugin_name = plugin_name.replace("_", "-")
+                if normalized_plugin_name not in configured_plugins:
                     continue
 
                 ai_functions = plugin_adapter.get_ai_functions(capability_id)
-                logger.debug(f"AI functions for {capability_id}: {[f.name for f in ai_functions]}")
 
                 for ai_function in ai_functions:
                     # Convert plugin AIFunction to registry format
@@ -838,7 +851,6 @@ def register_ai_functions_from_capabilities():
                         return plugin_function_wrapper
 
                     wrapped_handler = create_wrapper(ai_function.handler)
-                    logger.debug(f"Registering AI function: {ai_function.name}")
                     registry.register_function(ai_function.name, wrapped_handler, schema)
                     plugin_functions_count += 1
 
@@ -851,3 +863,49 @@ def register_ai_functions_from_capabilities():
         logger.debug("Plugin system not available for AI function registration")
     except Exception as e:
         logger.error(f"Failed to register plugin AI functions: {e}", exc_info=True)
+
+    # Register system capabilities (completion tools for iterative agents)
+    try:
+        from agent.capabilities.manager import get_capability_executor
+
+        # Check if mark_goal_complete system capability is available
+        completion_capability = get_capability_executor("mark_goal_complete")
+        if completion_capability:
+            # Create AI function schema for the completion tool
+            completion_schema = {
+                "name": "mark_goal_complete",
+                "description": "Call this when the goal has been fully achieved with high confidence",
+                "parameters": {
+                    "type": "object",
+                    "properties": {
+                        "summary": {"type": "string", "description": "What was accomplished"},
+                        "confidence": {
+                            "type": "number",
+                            "description": "Confidence in completion (0-1)",
+                            "minimum": 0,
+                            "maximum": 1,
+                        },
+                        "tasks_completed": {
+                            "type": "array",
+                            "description": "List of individual tasks that were finished",
+                            "items": {"type": "string"},
+                        },
+                        "remaining_issues": {
+                            "type": "array",
+                            "description": "Any known limitations or edge cases",
+                            "items": {"type": "string"},
+                        },
+                    },
+                    "required": ["summary", "confidence"],
+                },
+            }
+
+            # Register with the function registry
+            registry.register_function("mark_goal_complete", completion_capability, completion_schema)
+            registered_count += 1
+            logger.info("Registered system capability: mark_goal_complete")
+        else:
+            logger.warning("System capability mark_goal_complete not found in capabilities manager")
+
+    except Exception as e:
+        logger.error(f"Failed to register system capabilities: {e}", exc_info=True)
